""" Utility functions and classes for SRP

Context : SRP
Module  : PM
Version : 1.1.0
Author  : Stefano Covino
Date    : 22/08/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (23/08/2012) First version.
        : (22/08/2013) Improvement.
"""


from Simple import Simple
from SRP.SRPMath.FastAngularDistance import FastAngularDistance


def minSimple ((s_AOFS,s_EOFS),taz,talt,oaz,oalt):
    caz, calt = Simple((taz,talt),(s_AOFS,s_EOFS))
    return FastAngularDistance(oaz,oalt,caz,calt).sum()



""" 

Context : SRP
Module  : SRP.REM
Version : 1.0.0
Author  : Stefano Covino
Date    : 24/08/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (24/08/2013) First version.
"""


import math

from SRP.SRPFits.GetHeader import GetHeader
from SRP.SRPFits.GetWCS import GetWCS
from SRP.SRPREM.GetREMSite import GetREMSite
from SRP.SRPREM.GetObj import GetObj
import SRP.SRPREM as SREM


res = {'RA' : 0.0, 'DEC' : 0.0, 'RAOff' : 0.0, 'DECOff' : 0.0, 'AZ' : 0.0, 'ALT' : 0.0, 'AZOff' : 0.0, 'ALTOff' : 0.0}


def GetREMOffsets (filename, center=None):
    head = GetHeader(filename)[0]
    wcs = GetWCS(filename)[0]
    if head == None or wcs == None:
        res['RA'] = None
        res['DEC'] = None
        res['AZ'] = None
        res['ALT'] = None
        return res
    #
    try:
        RA = head[SREM.RA]
        DEC = head[SREM.DEC]
        dobs = head[SREM.DOBS]
    except KeyError:
        res['RA'] = None
        res['DEC'] = None
        res['AZ'] = None
        res['ALT'] = None
        return res    
    #
    if center == None:
        RAc,DECc = wcs.getCentreWCSCoords()
    else:
        RAc,DECc = wcs.pix2wcs(center['x'],center['y'])
    #
    site = GetREMSite()
    site.date = dobs.replace('T',' ').replace('-','/')
    #
    nb = GetObj(RA,DEC)
    nb.compute(site)
    AZ = math.degrees(float(nb.az))
    ALT = math.degrees(float(nb.alt))
    #
    nb = GetObj(RAc,DECc)
    nb.compute(site)
    AZc = math.degrees(float(nb.az))
    ALTc = math.degrees(float(nb.alt))
    #
    res['RA'] = RA
    res['DEC'] = DEC
    res['RAOff'] = RA-RAc
    res['DECOff'] = DEC-DECc    
    res['AZ'] = AZ
    res['ALT'] = ALT
    res['AZOff'] = AZ-AZc
    res['ALTOff'] = ALT-ALTc
    #
    return res    
    


""" Utility functions and classes for SRP

Context : SRP
Module  : PM
Version : 1.1.0
Author  : Stefano Covino
Date    : 07/01/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (28/08/2012) First version.
        : (03/12/2013) Better naming.
        : (07/01/2014) Better organization of parameters.
"""

import numpy
from Full import Full


def Extra ((az,alt),(e_AAN,e_EAN,e_AAE,e_EAE,e_NPAE,e_BNP,e_AES,e_AEC,e_EES,e_EEC,e_AOFS,e_EOFS,e_ES2A,e_EC2A,e_ES3A,e_EC3A,e_C5)):
    # AAN: error in the leveling of the telescope toward north
    # AAE: error in the leveling of the telescope toward east
    # NPAE: non-perpendicularity of the azimuth and elevation axis
    # BNP: non-perpendicularity of the optical and the elevation axis
    # AES, AEC, EES, EEC:  eccentricity of the encoders
    # AOFS: azimuth zero point correction
    # EOFS: altitude zero point correction
    # EAN: Azimut axis at N of vertical (Elevation correction, = AAN)
    # AAE: Azimut axis at E of vertical (Azimut correction, = EAE)
    # ES2A = C1: Irregularities in the bearing race
    # EC2A = C2: Irregularities in the bearing race
    # ES3A = C3: Irregularities in the bearing race
    # EC3A = C4: Irregularities in the bearing race
    # C5: Empirical term
    # TF: Mechanic Flexure (= EEC)

    naz, nalt = Full((az,alt),(e_AAN,e_EAN,e_AAE,e_EAE,e_NPAE,e_BNP,e_AES,e_AEC,e_EES,e_EEC,e_AOFS,e_EOFS))
    nalt = nalt + e_ES2A * numpy.sin(numpy.radians(2*az)) + e_EC2A * numpy.cos(numpy.radians(2*az)) + e_ES3A * numpy.sin(numpy.radians(3*az)) + e_EC3A * numpy.cos(numpy.radians(3*az)) + e_C5 / numpy.cos(numpy.radians(alt))
    return naz, nalt




""" Utility functions and classes for SRP

Context : SRP
Module  : PM
Version : 1.2.0
Author  : Stefano Covino
Date    : 07/01/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (22/08/2012) First version.
        : (01/07/2013) Check of parameter names.
        : (22/08/2013) Just offsets.
        : (03/12/2013) Better naming of parameters.
        : (07/01/2014) Better organization of parameters.
"""

import numpy


def Full ((az,alt),(f_AAN,f_EAN,f_AAE,f_EAE,f_NPAE,f_BNP,f_AES,f_AEC,f_EES,f_EEC,f_AOFS,f_EOFS)):
    # AAN: error in the leveling of the telescope toward north
    # EAE: error in the leveling of the telescope toward east
    # NPAE: non-perpendicularity of the azimuth and elevation axis
    # BNP: non-perpendicularity of the optical and the elevation axis
    # AES, AEC, EES, EEC:  eccentricity of the encoders
    # AOFS: azimuth zero point correction
    # EOFS: altitude zero point correction
    naz = f_AAN * numpy.sin(numpy.radians(az)) * numpy.tan(numpy.radians(alt)) - f_AAE * numpy.cos(numpy.radians(az)) * numpy.tan(numpy.radians(alt)) + f_NPAE * numpy.tan(numpy.radians(alt))  - f_BNP / numpy.cos(numpy.radians(alt)) + f_AOFS + f_AES * numpy.sin(numpy.radians(az)) + f_AEC * numpy.cos(numpy.radians(az))
    nalt = f_EAN * numpy.cos(numpy.radians(az)) + f_EAE * numpy.sin(numpy.radians(az)) + f_EOFS + f_EES * numpy.sin(numpy.radians(alt)) + f_EEC * numpy.cos(numpy.radians(alt))
    return naz, nalt




""" Utility functions and classes for SRP

Context : SRP
Module  : PM
Version : 1.0.0
Author  : Stefano Covino
Date    : 24/08/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (24/08/2012) First version.

"""

import numpy

TOLAZ = 1e-5
TOLALT = 1e-5
MAXITER = 100

def FindOrigCoord (az,alt,func,pars):
    offaz = 1
    offalt = 1
    niter = 0
    taz = az
    talt = alt
    while numpy.all(offaz > TOLAZ) and numpy.all(offalt > TOLALT) and niter <= MAXITER:
        naz,nalt = func((taz,talt),pars)
        #print naz,nalt,az,alt
        offaz = numpy.absolute(naz-az)
        offalt = numpy.absolute(nalt-alt)
        taz = numpy.where(naz >= az, taz - offaz/2., taz + offaz/2.)
        talt = numpy.where(nalt >= alt, talt - offalt/2., talt + offalt/2.)
        niter = niter + 1
    return taz, talt, niter <= MAXITER




""" Utility functions and classes for SRP

Context : SRP
Module  : PM
Version : 1.1.0
Author  : Stefano Covino
Date    : 22/08/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (22/08/2012) First version.
        : (22/08/2013) Just offsets.
"""

import numpy


def Classic ((az,alt),(c_AN,c_AE,c_NPAE,c_BNP,c_TF,c_AOFS,c_EOFS)):
    # AN: error in the leveling of the telescope toward north
    # AE: error in the leveling of the telescope toward east
    # NPAE: non-perpendicularity of the azimuth and elevation axis
    # BNP: non-perpendicularity of the optical and the elevation axis
    # TF: sagging of the tube
    # AOFS: azimuth zero point correction
    # EOFS: altitude zero point correction
    naz = c_AN * numpy.sin(numpy.radians(az)) * numpy.tan(numpy.radians(alt)) - c_AE * numpy.cos(numpy.radians(az)) * numpy.tan(numpy.radians(alt)) + c_NPAE * numpy.tan(numpy.radians(alt)) - c_BNP / numpy.cos(numpy.radians(alt)) + c_AOFS
    nalt = c_AN * numpy.cos(numpy.radians(az)) + c_AE * numpy.sin(numpy.radians(az)) + c_TF * numpy.cos(numpy.radians(alt)) + c_EOFS
    return naz, nalt




""" get REM site information

Module  : SRPREM.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 31/08/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (31/08/2012) First version.

"""

import ephem
import SRP.SRPREM as SR


def GetREMSite ():
    site = ephem.Observer()
    site.lat = str(SR.REMLAT)
    site.long = str(SR.REMLONG)
    site.elevation = SR.REMHEIGHT
    return site


""" Utility functions and classes for SRP

Context : SRP
Module  : PM
Version : 1.1.0
Author  : Stefano Covino
Date    : 22/08/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (22/08/2012) First version.
        : (22/08/2013) Just offsets.
"""



def Simple ((az,alt),(s_AOFS,s_EOFS)):
    # AOFS: azimuth zero point correction
    # EOFS: altitude zero point correction
    naz = s_AOFS+az-az
    nalt = s_EOFS+alt-alt
    return naz, nalt




""" Utility functions and classes for SRP

Context : SRP
Module  : PM
Version : 1.2.0
Author  : Stefano Covino
Date    : 07/01/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (23/08/2012) First version.
        : (01/07/2013) Better check of paramter names.
        : (22/08/2013) Improvement.
        : (03/12/2013) Better naming.
        : (07/01/2014) Better organization of parameters.
"""

from Full import Full
from SRP.SRPMath.FastAngularDistance import FastAngularDistance



def minFull ((f_AAN,f_EAN,f_AAE,f_EAE,f_NPAE,f_BNP,f_AES,f_AEC,f_EES,f_EEC,f_AOFS,f_EOFS),taz,talt,oaz,oalt):
    caz, calt = Full((taz,talt),(f_AAN,f_EAN,f_AAE,f_EAE,f_NPAE,f_BNP,f_AES,f_AEC,f_EES,f_EEC,f_AOFS,f_EOFS))
    return FastAngularDistance(oaz,oalt,caz,calt).sum()




""" get OBJ site information

Module  : SRPREM.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 31/08/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (31/08/2012) First version.

"""

import ephem
from SRP.SRPMath.AstroCoordInput import AstroCoordInput


def GetObj (ra,dec):
    coords = AstroCoordInput(ra,dec)
    nb = ephem.FixedBody()
    nb._ra = str(coords.EphemCoord.ra)
    nb._dec = str(coords.EphemCoord.dec)
    nb._epoch = ephem.date(ephem.J2000)
    return nb


""" 

Context : SRP
Module  : REM
Version : 1.1.0
Author  : Stefano Covino
Date    : 07/01/2014
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (24/08/2012) First version.
        : (22/08/2013) Funtions reordered.
        : (28/08/2013) New functions.
        : (16/09/2013) New functions.
        : (07/01/2014) Better organization of functions.
"""


__all__ = ['Classic', 'Extra', 'ExtraExt', 'FindOrigCoord', 'Full', 'minClassic', 'minExtra', 'minFull',
                'minSimple', 'Simple']


PMFlavours = ['classic', 'extra', 'full', 'simple']


""" Utility functions and classes for SRP

Context : SRP
Module  : PM
Version : 1.1.0
Author  : Stefano Covino
Date    : 07/01/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (28/08/2013) First version.
        : (03/12/2013) Better naming.
        : (07/01/2014) Better organization of parameters.
"""

from Extra import Extra
from SRP.SRPMath.FastAngularDistance import FastAngularDistance



def minExtra ((e_AAN,e_EAN,e_AAE,e_EAE,e_NPAE,e_BNP,e_AES,e_AEC,e_EES,e_EEC,e_AOFS,e_EOFS,e_ES2A,e_EC2A,e_ES3A,e_EC3A,e_C5),taz,talt,oaz,oalt):
    caz, calt = Extra((taz,talt),(e_AAN,e_EAN,e_AAE,e_EAE,e_NPAE,e_BNP,e_AES,e_AEC,e_EES,e_EEC,e_AOFS,e_EOFS,e_ES2A,e_EC2A,e_ES3A,e_EC3A,e_C5))
    return FastAngularDistance(oaz,oalt,caz,calt).sum()

